@@ -53,6 +53,20 @@ enum touch_state {
 	TOUCH_END = 5,
 };
 
+static inline const char *
+touch_state_to_str(enum touch_state state)
+{
+	switch(state) {
+	CASE_RETURN_STRING(TOUCH_NONE);
+	CASE_RETURN_STRING(TOUCH_HOVERING);
+	CASE_RETURN_STRING(TOUCH_BEGIN);
+	CASE_RETURN_STRING(TOUCH_UPDATE);
+	CASE_RETURN_STRING(TOUCH_MAYBE_END);
+	CASE_RETURN_STRING(TOUCH_END);
+	}
+	return NULL;
+}
+
 enum touch_palm_state {
 	PALM_NONE = 0,
 	PALM_EDGE,
@@ -92,17 +106,31 @@ enum tp_tap_state {
 	TAP_STATE_IDLE = 4,
 	TAP_STATE_TOUCH,
 	TAP_STATE_HOLD,
-	TAP_STATE_TAPPED,
+	TAP_STATE_1FGTAP_TAPPED,
+	TAP_STATE_2FGTAP_TAPPED,
+	TAP_STATE_3FGTAP_TAPPED,
 	TAP_STATE_TOUCH_2,
 	TAP_STATE_TOUCH_2_HOLD,
 	TAP_STATE_TOUCH_2_RELEASE,
 	TAP_STATE_TOUCH_3,
 	TAP_STATE_TOUCH_3_HOLD,
-	TAP_STATE_DRAGGING_OR_DOUBLETAP,
-	TAP_STATE_DRAGGING_OR_TAP,
-	TAP_STATE_DRAGGING,
-	TAP_STATE_DRAGGING_WAIT,
-	TAP_STATE_DRAGGING_2,
+	TAP_STATE_TOUCH_3_RELEASE,
+	TAP_STATE_TOUCH_3_RELEASE_2,
+	TAP_STATE_1FGTAP_DRAGGING_OR_DOUBLETAP,
+	TAP_STATE_2FGTAP_DRAGGING_OR_DOUBLETAP,
+	TAP_STATE_3FGTAP_DRAGGING_OR_DOUBLETAP,
+	TAP_STATE_1FGTAP_DRAGGING_OR_TAP,
+	TAP_STATE_2FGTAP_DRAGGING_OR_TAP,
+	TAP_STATE_3FGTAP_DRAGGING_OR_TAP,
+	TAP_STATE_1FGTAP_DRAGGING,
+	TAP_STATE_2FGTAP_DRAGGING,
+	TAP_STATE_3FGTAP_DRAGGING,
+	TAP_STATE_1FGTAP_DRAGGING_WAIT,
+	TAP_STATE_2FGTAP_DRAGGING_WAIT,
+	TAP_STATE_3FGTAP_DRAGGING_WAIT,
+	TAP_STATE_1FGTAP_DRAGGING_2,
+	TAP_STATE_2FGTAP_DRAGGING_2,
+	TAP_STATE_3FGTAP_DRAGGING_2,
 	TAP_STATE_DEAD, /**< finger count exceeded */
 };
 
@@ -158,6 +186,7 @@ struct tp_touch {
 	bool dirty;
 	struct device_coords point;
 	uint64_t time;
+	uint64_t initial_time;
 	int pressure;
 	bool is_tool_palm; /* MT_TOOL_PALM */
 	int major, minor;
@@ -280,6 +309,7 @@ struct tp_dispatch {
 	unsigned int fake_touches;
 
 	struct {
+		bool detection_disabled;
 		struct ratelimit warning;
 	} jump;
 
@@ -624,7 +654,7 @@ tp_button_is_inside_softbutton_area(const struct tp_dispatch *tp,
 
 void
 tp_release_all_taps(struct tp_dispatch *tp,
-		    uint64_t time);
+		    uint64_t now);
 
 void
 tp_tap_suspend(struct tp_dispatch *tp, uint64_t time);
