@@ -546,6 +546,52 @@ START_TEST(evcode_prop_parser)
 }
 END_TEST
 
+START_TEST(input_prop_parser)
+{
+	struct parser_test_val {
+		const char *prop;
+		bool success;
+		size_t nvals;
+		uint32_t values[20];
+	} tests[] = {
+		{ "INPUT_PROP_BUTTONPAD", true, 1, {INPUT_PROP_BUTTONPAD}},
+		{ "INPUT_PROP_BUTTONPAD;INPUT_PROP_POINTER", true, 2,
+			{ INPUT_PROP_BUTTONPAD,
+			  INPUT_PROP_POINTER }},
+		{ "INPUT_PROP_BUTTONPAD;0x00;0x03", true, 3,
+			{ INPUT_PROP_BUTTONPAD,
+			  INPUT_PROP_POINTER,
+			  INPUT_PROP_SEMI_MT }},
+		{ .prop = "", .success = false },
+		{ .prop = "0xff", .success = false },
+		{ .prop = "INPUT_PROP", .success = false },
+		{ .prop = "INPUT_PROP_FOO", .success = false },
+		{ .prop = "INPUT_PROP_FOO;INPUT_PROP_FOO", .success = false },
+		{ .prop = "INPUT_PROP_POINTER;INPUT_PROP_FOO", .success = false },
+		{ .prop = "none", .success = false },
+		{ .prop = NULL },
+	};
+	struct parser_test_val *t;
+
+	for (int i = 0; tests[i].prop; i++) {
+		bool success;
+		uint32_t props[32];
+		size_t nprops = ARRAY_LENGTH(props);
+
+		t = &tests[i];
+		success = parse_input_prop_property(t->prop, props, &nprops);
+		ck_assert(success == t->success);
+		if (!success)
+			continue;
+
+		ck_assert_int_eq(nprops, t->nvals);
+		for (size_t j = 0; j < t->nvals; j++) {
+			ck_assert_int_eq(t->values[j], props[j]);
+		}
+	}
+}
+END_TEST
+
 START_TEST(evdev_abs_parser)
 {
 	struct test {
@@ -633,6 +679,37 @@ START_TEST(time_conversion)
 }
 END_TEST
 
+START_TEST(human_time)
+{
+	struct ht_tests {
+		uint64_t interval;
+		unsigned int value;
+		const char *unit;
+	} tests[] = {
+		{ 0, 0, "us" },
+		{ 123, 123, "us" },
+		{ ms2us(5), 5, "ms" },
+		{ ms2us(100), 100, "ms" },
+		{ s2us(5), 5, "s" },
+		{ s2us(100), 100, "s" },
+		{ s2us(120), 2, "min" },
+		{ 5 * s2us(60), 5, "min" },
+		{ 120 * s2us(60), 2, "h" },
+		{ 5 * 60 * s2us(60), 5, "h" },
+		{ 48 * 60 * s2us(60), 2, "d" },
+		{ 1000 * 24 * 60 * s2us(60), 1000, "d" },
+		{ 0, 0, NULL },
+	};
+	for (int i = 0; tests[i].unit != NULL; i++) {
+		struct human_time ht;
+
+		ht = to_human_time(tests[i].interval);
+		ck_assert_int_eq(ht.value, tests[i].value);
+		ck_assert_str_eq(ht.unit, tests[i].unit);
+	}
+}
+END_TEST
+
 struct atoi_test {
 	char *str;
 	bool success;
@@ -1071,6 +1148,54 @@ START_TEST(strstrip_test)
 }
 END_TEST
 
+START_TEST(strendswith_test)
+{
+	struct strendswith_test {
+		const char *string;
+		const char *suffix;
+		bool expected;
+	} tests[] = {
+		{ "foobar", "bar", true },
+		{ "foobar", "foo", false },
+		{ "foobar", "foobar", true },
+		{ "foo", "foobar", false },
+		{ "foobar", "", false },
+		{ "", "", false },
+		{ "", "foo", false },
+		{ NULL, NULL, false },
+	};
+
+	for (struct strendswith_test *t = tests; t->string; t++) {
+		ck_assert_int_eq(strendswith(t->string, t->suffix),
+				 t->expected);
+	}
+}
+END_TEST
+
+START_TEST(strstartswith_test)
+{
+	struct strstartswith_test {
+		const char *string;
+		const char *suffix;
+		bool expected;
+	} tests[] = {
+		{ "foobar", "foo", true },
+		{ "foobar", "bar", false },
+		{ "foobar", "foobar", true },
+		{ "foo", "foobar", false },
+		{ "foo", "", false },
+		{ "", "", false },
+		{ "foo", "", false },
+		{ NULL, NULL, false },
+	};
+
+	for (struct strstartswith_test *t = tests; t->string; t++) {
+		ck_assert_int_eq(strstartswith(t->string, t->suffix),
+				 t->expected);
+	}
+}
+END_TEST
+
 START_TEST(list_test_insert)
 {
 	struct list_test {
@@ -1145,6 +1270,54 @@ START_TEST(strverscmp_test)
 }
 END_TEST
 
+START_TEST(streq_test)
+{
+	ck_assert(streq("", "") == true);
+	ck_assert(streq(NULL, NULL) == true);
+	ck_assert(streq("0.0.1", "") == false);
+	ck_assert(streq("foo", NULL) == false);
+	ck_assert(streq(NULL, "foo") == false);
+	ck_assert(streq("0.0.1", "0.0.1") == true);
+}
+END_TEST
+
+START_TEST(strneq_test)
+{
+	ck_assert(strneq("", "", 1) == true);
+	ck_assert(strneq(NULL, NULL, 1) == true);
+	ck_assert(strneq("0.0.1", "", 6) == false);
+	ck_assert(strneq("foo", NULL, 5) == false);
+	ck_assert(strneq(NULL, "foo", 5) == false);
+	ck_assert(strneq("0.0.1", "0.0.1", 6) == true);
+}
+END_TEST
+
+START_TEST(trunkname_test)
+{
+	struct test {
+		const char *path;
+		const char *expected;
+	} tests[] = {
+		{ "foo.c", "foo" },
+		{ "/path/to/foo.h", "foo" },
+		{ "../bar.foo", "bar" },
+		{ "./bar.foo.baz", "bar.foo" },
+		{ "./", "" },
+		{ "/", "" },
+		{ "/bar/", "" },
+		{ "/bar", "bar" },
+		{ "", "" },
+	};
+	struct test *t;
+
+	ARRAY_FOR_EACH(tests, t) {
+		char *result = trunkname(t->path);
+		ck_assert_str_eq(result, t->expected);
+		free(result);
+	}
+}
+END_TEST
+
 static Suite *
 litest_utils_suite(void)
 {
@@ -1165,6 +1338,7 @@ litest_utils_suite(void)
 	tcase_add_test(tc, calibration_prop_parser);
 	tcase_add_test(tc, range_prop_parser);
 	tcase_add_test(tc, evcode_prop_parser);
+	tcase_add_test(tc, input_prop_parser);
 	tcase_add_test(tc, evdev_abs_parser);
 	tcase_add_test(tc, safe_atoi_test);
 	tcase_add_test(tc, safe_atoi_base_16_test);
@@ -1177,11 +1351,17 @@ litest_utils_suite(void)
 	tcase_add_test(tc, kvsplit_double_test);
 	tcase_add_test(tc, strjoin_test);
 	tcase_add_test(tc, strstrip_test);
+	tcase_add_test(tc, strendswith_test);
+	tcase_add_test(tc, strstartswith_test);
 	tcase_add_test(tc, time_conversion);
+	tcase_add_test(tc, human_time);
 
 	tcase_add_test(tc, list_test_insert);
 	tcase_add_test(tc, list_test_append);
 	tcase_add_test(tc, strverscmp_test);
+	tcase_add_test(tc, streq_test);
+	tcase_add_test(tc, strneq_test);
+	tcase_add_test(tc, trunkname_test);
 
 	suite_add_tcase(s, tc);
 
