@@ -29,6 +29,7 @@
 #include <libevdev/libevdev.h>
 #include <libudev.h>
 #include <sys/signalfd.h>
+#include <sys/timerfd.h>
 #include <sys/utsname.h>
 #include <sys/stat.h>
 #include <string.h>
@@ -1956,12 +1957,16 @@ select_device(void)
 	char *device_path;
 	bool has_eaccess = false;
 	int available_devices = 0;
+	const char *prefix = "";
+
+	if (!isatty(STDERR_FILENO))
+		prefix = "# ";
 
 	ndev = scandir("/dev/input", &namelist, is_event_node, versionsort);
 	if (ndev <= 0)
 		return NULL;
 
-	fprintf(stderr, "Available devices:\n");
+	fprintf(stderr, "%sAvailable devices:\n", prefix);
 	for (int i = 0; i < ndev; i++) {
 		struct libevdev *device;
 		char path[PATH_MAX];
@@ -1983,7 +1988,7 @@ select_device(void)
 		if (rc != 0)
 			continue;
 
-		fprintf(stderr, "%s:	%s\n", path, libevdev_get_name(device));
+		fprintf(stderr, "%s%s:	%s\n", prefix, path, libevdev_get_name(device));
 		libevdev_free(device);
 		available_devices++;
 	}
@@ -1993,14 +1998,13 @@ select_device(void)
 	free(namelist);
 
 	if (available_devices == 0) {
-		fprintf(stderr, "No devices available. ");
-		if (has_eaccess)
-				fprintf(stderr, "Please re-run as root.");
-		fprintf(stderr, "\n");
+		fprintf(stderr,
+			"No devices available.%s\n",
+			has_eaccess ? " Please re-run as root." : "");
 		return NULL;
 	}
 
-	fprintf(stderr, "Select the device event number: ");
+	fprintf(stderr, "%sSelect the device event number: ", prefix);
 	rc = scanf("%d", &selected_device);
 
 	if (rc != 1 || selected_device < 0)
@@ -2107,15 +2111,41 @@ print_progress_bar(void)
 	fprintf(stderr, "\rReceiving events: [%*s%*s]", foo, "*", 21 - foo, " ");
 }
 
+static inline void
+print_wall_time(struct record_context *ctx)
+{
+	time_t t = time(NULL);
+	struct tm tm;
+
+	localtime_r(&t, &tm);
+	iprintf(ctx, "# Current time is %02d:%02d:%02d\n", tm.tm_hour, tm.tm_min, tm.tm_sec);
+}
+
+static inline void
+arm_timer(int timerfd)
+{
+	time_t t = time(NULL);
+	struct tm tm;
+	struct itimerspec interval = {
+		.it_value = { 0, 0 },
+		.it_interval = { 5, 0 },
+	};
+
+	localtime_r(&t, &tm);
+	interval.it_value.tv_sec = 5 - (tm.tm_sec % 5);
+	timerfd_settime(timerfd, 0, &interval, NULL);
+}
+
 static int
 mainloop(struct record_context *ctx)
 {
 	bool autorestart = (ctx->timeout > 0);
-	struct pollfd fds[ctx->ndevices + 2];
+	struct pollfd fds[ctx->ndevices + 3];
+	struct pollfd *signal_fd = &fds[0];
+	struct pollfd *time_fd = &fds[1];
+	struct pollfd *libinput_fd = NULL;
 	unsigned int nfds = 0;
 	struct record_device *d = NULL;
-	struct record_device *first_device = NULL;
-	struct timespec ts;
 	sigset_t mask;
 
 	assert(ctx->timeout != 0);
@@ -2126,24 +2156,30 @@ mainloop(struct record_context *ctx)
 	sigaddset(&mask, SIGQUIT);
 	sigprocmask(SIG_BLOCK, &mask, NULL);
 
-	fds[0].fd = signalfd(-1, &mask, SFD_NONBLOCK);
-	fds[0].events = POLLIN;
-	fds[0].revents = 0;
-	assert(fds[0].fd != -1);
+	for (size_t i = 0; i < ARRAY_LENGTH(fds); i++) {
+		fds[i] = (struct pollfd) {
+			.fd = -1,
+			.events = POLLIN,
+			.revents = 0,
+		};
+	}
+
+	signal_fd->fd = signalfd(-1, &mask, SFD_NONBLOCK);
+	assert(signal_fd->fd != -1);
+	nfds++;
+
+	time_fd->fd = timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
+	assert(time_fd->fd != -1);
+	arm_timer(time_fd->fd);
 	nfds++;
 
 	if (ctx->libinput) {
-		fds[1].fd = libinput_get_fd(ctx->libinput);
-		fds[1].events = POLLIN;
-		fds[1].revents = 0;
-		nfds++;
-		assert(nfds == 2);
+		libinput_fd = &fds[nfds++];
+		libinput_fd->fd = libinput_get_fd(ctx->libinput);
 	}
 
 	list_for_each(d, &ctx->devices, link) {
 		fds[nfds].fd = libevdev_get_fd(d->evdev);
-		fds[nfds].events = POLLIN;
-		fds[nfds].revents = 0;
 		assert(fds[nfds].fd != -1);
 		nfds++;
 	}
@@ -2152,6 +2188,8 @@ mainloop(struct record_context *ctx)
 	 * start time. Otherwise, the first event starts the recording time.
 	 */
 	if (ctx->ndevices > 1) {
+		struct timespec ts;
+
 		clock_gettime(CLOCK_MONOTONIC, &ts);
 		ctx->offset = s2us(ts.tv_sec) + ns2us(ts.tv_nsec);
 	}
@@ -2159,6 +2197,9 @@ mainloop(struct record_context *ctx)
 	do {
 		int rc;
 		bool had_events = false; /* we delete files without events */
+		bool had_events_since_last_time = false;
+		bool skipped_timer_print = false;
+		struct record_device *first_device = NULL;
 
 		if (!open_output_file(ctx, autorestart)) {
 			fprintf(stderr,
@@ -2166,7 +2207,9 @@ mainloop(struct record_context *ctx)
 				ctx->output_file);
 			break;
 		}
-		fprintf(stderr, "Recording to '%s'.\n", ctx->output_file);
+		fprintf(stderr, "%sRecording to '%s'.\n",
+			isatty(STDERR_FILENO) ? "" : "# ",
+			ctx->output_file);
 
 		print_header(ctx);
 		if (autorestart)
@@ -2184,6 +2227,7 @@ mainloop(struct record_context *ctx)
 						link);
 		print_device_description(ctx, first_device);
 
+		print_wall_time(ctx);
 		iprintf(ctx, "events:\n");
 		indent_push(ctx);
 
@@ -2200,16 +2244,42 @@ mainloop(struct record_context *ctx)
 				fprintf(stderr, "Error: %m\n");
 				autorestart = false;
 				break;
-			} else if (rc == 0) {
+			}
+
+			if (rc == 0) {
 				fprintf(stderr,
 					" ... timeout%s\n",
 					had_events ? "" : " (file is empty)");
 				break;
-			} else if (fds[0].revents != 0) { /* signal */
+
+			}
+
+			if (signal_fd->revents != 0) { /* signal */
 				autorestart = false;
 				break;
 			}
 
+			if (time_fd->revents) { /* timer expiry */
+				char discard[64];
+				read(time_fd->fd, discard, sizeof(discard));
+
+				if (had_events_since_last_time) {
+					print_wall_time(ctx);
+					had_events_since_last_time = false;
+					skipped_timer_print = false;
+				} else {
+					skipped_timer_print = true;
+				}
+
+				if (rc == 1) /* no other fds have data */
+					continue;
+			}
+
+			if (skipped_timer_print) {
+				print_wall_time(ctx);
+				skipped_timer_print = false;
+			}
+
 			/* Pull off the evdev events first since they cause
 			 * libinput events.
 			 * handle_events de-queues libinput events so by the
@@ -2217,12 +2287,14 @@ mainloop(struct record_context *ctx)
 			 * events and libinput events roughly in sync.
 			 */
 			had_events = true;
-			list_for_each(d, &ctx->devices, link)
+			had_events_since_last_time = true;
+			list_for_each(d, &ctx->devices, link) {
 				handle_events(ctx, d, d == first_device);
+			}
 
 			/* This shouldn't pull any events off unless caused
 			 * by libinput-internal timeouts (e.g. tapping) */
-			if (ctx->libinput && fds[1].revents) {
+			if (ctx->libinput && libinput_fd->revents) {
 				size_t count, offset;
 
 				libinput_dispatch(ctx->libinput);
@@ -2282,7 +2354,8 @@ mainloop(struct record_context *ctx)
 		ctx->output_file = NULL;
 	} while (autorestart);
 
-	close(fds[0].fd);
+	close(signal_fd->fd);
+	close(time_fd->fd);
 
 	sigprocmask(SIG_UNBLOCK, &mask, NULL);
 
@@ -2290,7 +2363,7 @@ mainloop(struct record_context *ctx)
 }
 
 static inline bool
-init_device(struct record_context *ctx, char *path)
+init_device(struct record_context *ctx, char *path, bool grab)
 {
 	struct record_device *d;
 	int fd, rc;
@@ -2320,6 +2393,17 @@ init_device(struct record_context *ctx, char *path)
 		goto error;
 	}
 
+	if (grab) {
+		rc = libevdev_grab(d->evdev, LIBEVDEV_GRAB);
+		if (rc != 0) {
+			fprintf(stderr,
+				"Grab failed on %s: %s\n",
+				path,
+				strerror(-rc));
+			goto error;
+		}
+	}
+
 	libevdev_set_clock_id(d->evdev, CLOCK_MONOTONIC);
 
 	if (libevdev_get_num_slots(d->evdev) > 0)
@@ -2441,6 +2525,7 @@ enum options {
 	OPT_MULTIPLE,
 	OPT_ALL,
 	OPT_LIBINPUT,
+	OPT_GRAB,
 };
 
 int
@@ -2458,11 +2543,12 @@ main(int argc, char **argv)
 		{ "all", no_argument, 0, OPT_ALL },
 		{ "help", no_argument, 0, OPT_HELP },
 		{ "with-libinput", no_argument, 0, OPT_LIBINPUT },
+		{ "grab", no_argument, 0, OPT_GRAB },
 		{ 0, 0, 0, 0 },
 	};
 	struct record_device *d, *tmp;
 	const char *output_arg = NULL;
-	bool all = false, with_libinput = false;
+	bool all = false, with_libinput = false, grab = false;
 	int ndevices;
 	int rc = EXIT_FAILURE;
 
@@ -2506,6 +2592,9 @@ main(int argc, char **argv)
 		case OPT_LIBINPUT:
 			with_libinput = true;
 			break;
+		case OPT_GRAB:
+			grab = true;
+			break;
 		default:
 			usage();
 			rc = EXIT_INVALID_USAGE;
@@ -2603,7 +2692,7 @@ main(int argc, char **argv)
 		d = devices;
 
 		while (*d) {
-			if (!init_device(&ctx, safe_strdup(*d))) {
+			if (!init_device(&ctx, safe_strdup(*d), grab)) {
 				strv_free(devices);
 				goto out;
 			}
@@ -2622,7 +2711,7 @@ main(int argc, char **argv)
 		for (int i = ndevices; i > 0; i -= 1) {
 			char *devnode = safe_strdup(argv[optind + i - 1]);
 
-			if (!init_device(&ctx, devnode))
+			if (!init_device(&ctx, devnode, grab))
 				goto out;
 		}
 	} else {
@@ -2633,7 +2722,7 @@ main(int argc, char **argv)
 			goto out;
 		}
 
-		if (!init_device(&ctx, path))
+		if (!init_device(&ctx, path, grab))
 			goto out;
 	}
 
